@@ -148,11 +148,16 @@ void buffer_commit(buffer *b, size_t size)
 	size_t sz = b->used;
 	if (0 == sz) sz = 1;
 
+  #if __has_builtin(__builtin_add_overflow)
+	if (__builtin_add_overflow(size, sz, &sz))
+		ck_assert_failed(__FILE__, __LINE__, "add overflow");
+  #else
 	if (size > 0) {
 		/* check for overflow: unsigned overflow is defined to wrap around */
 		sz += size;
 		force_assert(sz > size);
 	}
+  #endif
 
 	b->used = sz;
 	b->ptr[sz - 1] = '\0';
@@ -233,8 +238,14 @@ void buffer_append_iovec(buffer * const restrict b, const struct const_iovec * c
 
 void buffer_append_path_len(buffer * restrict b, const char * restrict a, size_t alen) {
     char * restrict s = buffer_string_prepare_append(b, alen+1);
+  #ifdef _WIN32
+    const int aslash = (alen && (a[0] == '/' || a[0] == '\\'));
+    if (b->used > 1 && (s[-1] == '/' || s[-1] == '\\'))
+  #else
     const int aslash = (alen && a[0] == '/');
-    if (b->used > 1 && s[-1] == '/') {
+    if (b->used > 1 && s[-1] == '/')
+  #endif
+    {
         if (aslash) {
             ++a;
             --alen;
@@ -436,25 +447,21 @@ int buffer_is_equal(const buffer *a, const buffer *b) {
 
 
 void li_tohex_lc(char * const restrict buf, size_t buf_len, const char * const restrict s, size_t s_len) {
-	force_assert(2 * s_len > s_len);
-	force_assert(2 * s_len < buf_len);
+	force_assert(s_len <= (buf_len >> 1));
 
 	for (size_t i = 0; i < s_len; ++i) {
 		buf[2*i]   = hex_chars_lc[(s[i] >> 4) & 0x0F];
 		buf[2*i+1] = hex_chars_lc[s[i] & 0x0F];
 	}
-	buf[2*s_len] = '\0';
 }
 
 void li_tohex_uc(char * const restrict buf, size_t buf_len, const char * const restrict s, size_t s_len) {
-	force_assert(2 * s_len > s_len);
-	force_assert(2 * s_len < buf_len);
+	force_assert(s_len <= (buf_len >> 1));
 
 	for (size_t i = 0; i < s_len; ++i) {
 		buf[2*i]   = hex_chars_uc[(s[i] >> 4) & 0x0F];
 		buf[2*i+1] = hex_chars_uc[s[i] & 0x0F];
 	}
-	buf[2*s_len] = '\0';
 }
 
 
@@ -900,7 +907,7 @@ void buffer_path_simplify(buffer *b)
         return;
     }
 
-  #if defined(__WIN32) || defined(__CYGWIN__)
+  #if defined(_WIN32) || defined(__CYGWIN__)
     /* cygwin is treating \ and / the same, so we have to that too */
     for (char *p = b->ptr; *p; p++) {
         if (*p == '\\') *p = '/';
