@@ -37,7 +37,8 @@ struct fdnode_st {
 #define FDEVENT_ERR    0x0008
 #define FDEVENT_HUP    0x0010
 #define FDEVENT_NVAL   0x0020
-#if defined(__sun) && defined(__SVR4) /* Solaris */
+#if (defined(__sun) && defined(__SVR4)) /* Solaris */ \
+ || defined(__FreeBSD__)
 #define FDEVENT_RDHUP  0x4000
 #else
 #define FDEVENT_RDHUP  0x2000
@@ -115,6 +116,7 @@ int fdevent_open_dirname(char *path, int symlinks);
 #ifndef _WIN32
 int fdevent_set_stdin_stdout_stderr(int fdin, int fdout, int fderr);
 pid_t fdevent_fork_execve(const char *name, char *argv[], char *envp[], int fdin, int fdout, int fderr, int dfd);
+pid_t fdevent_sh_exec(const char *cmdstr, char *envp[], int fdin, int fdout, int fderr);
 #endif
 pid_t fdevent_waitpid(pid_t pid, int *status, int nb);
 pid_t fdevent_waitpid_intr(pid_t pid, int *status);
@@ -123,9 +125,9 @@ int fdevent_kill(pid_t pid, int sig);
 #ifdef _WIN32
 __attribute_cold__
 void fdevent_win32_cleanup (void);
-
 #define fdevent_fork_execve(name, argv, envp, fdin, fdout, fderr, dfd) \
         fdevent_createprocess((argv),(envp),(fdin),(fdout),(fderr),(dfd))
+pid_t fdevent_sh_exec(const char *cmdstr, char *envp[], intptr_t fdin, intptr_t fdout, int fderr);
 pid_t fdevent_createprocess(char *argv[], char *envp[], intptr_t fdin, intptr_t fdout, int fderr, int dfd);
 #endif /* _WIN32 */
 
