@@ -242,14 +242,14 @@ int fdevent_socketpair_nb_cloexec (int domain, int typ, int protocol, int sv[2])
 
 int fdevent_socket_set_cloexec (int fd)
 {
-    return SetHandleInformation((HANDLE)(uint64_t)fd,
+    return SetHandleInformation((HANDLE)(uintptr_t)fd,
                                 HANDLE_FLAG_INHERIT, 0) ? 0 : -1;
 }
 
 
 int fdevent_socket_clr_cloexec (int fd)
 {
-    return SetHandleInformation((HANDLE)(uint64_t)fd,
+    return SetHandleInformation((HANDLE)(uintptr_t)fd,
                                 HANDLE_FLAG_INHERIT, HANDLE_FLAG_INHERIT)
       ? 0
       : -1;
@@ -265,7 +265,7 @@ int fdevent_socket_set_nb (int fd)
 
 int fdevent_socket_set_nb_cloexec (int fd)
 {
-    return SetHandleInformation((HANDLE)(uint64_t)fd, HANDLE_FLAG_INHERIT, 0)
+    return SetHandleInformation((HANDLE)(uintptr_t)fd, HANDLE_FLAG_INHERIT, 0)
       ? fdevent_socket_set_nb(fd)
       : -1;
 }
@@ -331,6 +331,7 @@ int fdevent_fcntl_set_nb_cloexec_sock (int fd)
 
 #include <windows.h>
 #include <signal.h>     /* sig_atomic_t SIGKILL */
+#include <errno.h>
 
 #ifndef SIGKILL
 #define SIGKILL 9
@@ -554,6 +555,7 @@ pid_t fdevent_createprocess (char *argv[], char *envp[], intptr_t fdin, intptr_t
     char *dirp = NULL;
     if (0 == strcmp(argv[0],"/bin/sh") && argv[1] && 0 == strcmp(argv[1],"-c")){
         /* future: consider checking for SHELL variable in environment */
+        /* future: getenv("COMSPEC") and use, if available, for cmd.exe */
       #if 1
         *(const char **)&argv[0] = "C:\\Windows\\System32\\cmd.exe";
         *(const char **)&argv[1] = "/c";
@@ -566,12 +568,26 @@ pid_t fdevent_createprocess (char *argv[], char *envp[], intptr_t fdin, intptr_t
         /* dfd == -2 for argv[0], dfd == -3 indicates argv[1] */
         const char *arg = (-3 == dfd) ? argv[1] : argv[0];
         if (arg && (arg[0] == '\\' || arg[0] == '/' || arg[1] == ':')) {
+            /* dirname must be a Windows path, not a cygwin or msys2 path.
+             * Attempt to remove the leading cygwin or msys2 driver signifier.
+             * Note: not handling /proc and does not work across volumes.
+             * future: getenv("CYGVOL") and do case-insensitive prefix match */
             /* step over "/cygdrive/c/..." (or other drive letter) -> "/..."
              * (note: below code assumes script is on current volume)
              * (to honor volume, could copy volume and replace '/' with ':') */
             if (0 == memcmp(arg, "/cygdrive/", 10)
                 && arg[10] != '\0' && arg[11] == '/')
                 arg += 11;
+            else if (arg[0] == '/' && arg[1] != '\0' && arg[2] == '/'
+                     && getenv("MSYSTEM")) { /* MSYS2 */
+                /* not perfect, but catches CYGVOL prepended in mod_cgi.c
+                 * and omitted in gw_backend.c as long as path from gw_backend.c
+                 * does not happen to start w/ same single letter folder name */
+                const char *cygvol = getenv("CYGVOL");
+                len = cygvol ? strlen(cygvol) : 0;
+                if (len && 0 == _strnicmp(arg, cygvol, len))
+                    arg += len;
+            }
 
             char *sl = strrchr(arg, '/');
             char *bs = strrchr(arg, '\\');
@@ -645,7 +661,7 @@ pid_t fdevent_createprocess (char *argv[], char *envp[], intptr_t fdin, intptr_t
      * Programmatically controlling which handles are inherited by new processes
      * in Win32: https://devblogs.microsoft.com/oldnewthing/20111216-00/?p=8873
      */
-    size_t sz = 0;
+    SIZE_T sz = 0;
     InitializeProcThreadAttributeList(NULL, 1, 0, &sz);
     /* GetLastError() == ERROR_INSUFFICIENT_BUFFER */
     LPPROC_THREAD_ATTRIBUTE_LIST attrlist = info.lpAttributeList = malloc(sz);
@@ -767,6 +783,18 @@ pid_t fdevent_createprocess (char *argv[], char *envp[], intptr_t fdin, intptr_t
 }
 
 
+pid_t fdevent_sh_exec (const char *cmdstr, char *envp[], intptr_t fdin, intptr_t fdout, int fderr)
+{
+    char *args[4];
+    const char *shell = getenv("SHELL");
+    *(const char **)&args[0] = shell ? shell : "/bin/sh";
+    *(const char **)&args[1] = "-c";
+    *(const char **)&args[2] = cmdstr;
+    args[3] = NULL;
+    return fdevent_createprocess(args, envp, fdin, fdout, fderr, -1);
+}
+
+
 int fdevent_dup_cloexec (int fd)
 {
     const int newfd = _dup(fd);
