@@ -13,16 +13,21 @@
 #include "http_date.h"
 #include "http_etag.h"
 #include "http_header.h"
-#include "response.h"
+#include "plugin_config.h" /* config_feature_bool */
 #include "sock_addr.h"
 #include "stat_cache.h"
 
+#undef __declspec_dllimport__
+#define __declspec_dllimport__  __declspec_dllexport__
+
+#include "response.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 
 #include "sys-socket.h"
-#include <unistd.h>
+#include "sys-unistd.h" /* <unistd.h> */
 
 /**
  * max size of the HTTP response header from backends
@@ -30,6 +35,10 @@
  */
 #define MAX_HTTP_RESPONSE_FIELD_SIZE 65535
 
+/* http_dispatch instance here to be defined in shared object (see base.h)*/
+__declspec_dllexport__
+struct http_dispatch http_dispatch[4];
+
 
 __attribute_cold__
 int http_response_buffer_append_authority(request_st * const r, buffer * const o) {
@@ -132,6 +141,45 @@ int http_response_redirect_to_directory(request_st * const r, int status) {
 	return 0;
 }
 
+
+__attribute_cold__
+void
+http_response_delay (connection * const con)
+{
+    if (config_feature_bool(con->srv, "auth.delay-invalid-creds", 1)){
+        /*(delay sending response)*/
+        con->is_writable = 0;
+        con->traffic_limit_reached = 1;
+    }
+}
+
+
+int
+http_response_omit_header (request_st * const r, const data_string * const ds)
+{
+    const size_t klen = buffer_clen(&ds->key);
+    if (klen == sizeof("X-Sendfile")-1
+        && buffer_eq_icase_ssn(ds->key.ptr, CONST_STR_LEN("X-Sendfile")))
+        return 1;
+    if (klen >= sizeof("X-LIGHTTPD-")-1
+        && buffer_eq_icase_ssn(ds->key.ptr, CONST_STR_LEN("X-LIGHTTPD-"))) {
+        if (klen == sizeof("X-LIGHTTPD-KBytes-per-second")-1
+            && buffer_eq_icase_ssn(ds->key.ptr+sizeof("X-LIGHTTPD-")-1,
+                                   CONST_STR_LEN("KBytes-per-second"))) {
+            /* "X-LIGHTTPD-KBytes-per-second" */
+            off_t limit = strtol(ds->value.ptr, NULL, 10) << 10; /*(*=1024)*/
+            if (limit > 0
+                && (limit < r->conf.bytes_per_second
+                    || 0 == r->conf.bytes_per_second)) {
+                r->conf.bytes_per_second = limit;
+            }
+        }
+        return 1;
+    }
+    return 0;
+}
+
+
 #define MTIME_CACHE_MAX 16
 struct mtime_cache_type {
     unix_time64_t mtime;  /* key */
@@ -188,6 +236,38 @@ static int http_response_maybe_cachable (const request_st * const r) {
 }
 
 
+static handler_t http_response_304 (request_st * const r) {
+  #if 0 /* option: could set Content-Length with 304 if content known */
+    /* RFC9110
+     * https://www.rfc-editor.org/rfc/rfc9110#name-content-length */
+    /*assert(r->resp_body_finished);*/
+    off_t qlen = chunkqueue_length(&r->write_queue);
+    if (qlen > 0)
+        buffer_append_int(
+          http_header_response_set_ptr(r, HTTP_HEADER_CONTENT_LENGTH,
+                                       CONST_STR_LEN("Content-Length")),
+          qlen);
+  #endif
+
+  #if 0 /* performed in http_response_write_prepare() */
+    http_response_body_clear(r, 1);
+    r->resp_body_finished = 1;
+  #endif
+    r->http_status = 304;
+    return HANDLER_FINISHED;
+}
+
+
+__attribute_cold__
+static handler_t http_response_412 (request_st * const r) {
+    /* http_response_reqbody_read_error() without setting r->keep_alive = 0 */
+    http_response_body_clear(r, 0);
+    r->handler_module = NULL;
+    r->http_status = 412;
+    return HANDLER_FINISHED;
+}
+
+
 int http_response_handle_cachable(request_st * const r, const buffer *lmod, const unix_time64_t lmtime) {
 	if (!http_response_maybe_cachable(r))
 		return HANDLER_GO_ON;
@@ -211,14 +291,9 @@ int http_response_handle_cachable(request_st * const r, const buffer *lmod, cons
 		/*(weak etag comparison must not be used for ranged requests)*/
 		int range_request = (0 != light_btst(r->rqst_htags, HTTP_HEADER_RANGE));
 		if (http_etag_matches(etag, vb->ptr, !range_request)) {
-			if (http_method_get_head_query(r->http_method)) {
-				r->http_status = 304;
-				return HANDLER_FINISHED;
-			} else {
-				r->http_status = 412;
-				r->handler_module = NULL;
-				return HANDLER_FINISHED;
-			}
+			return http_method_get_head_query(r->http_method)
+			  ? http_response_304(r)
+			  : http_response_412(r);
 		}
 	} else if (http_method_get_head_query(r->http_method)
 		   && (vb = http_header_request_get(r, HTTP_HEADER_IF_MODIFIED_SINCE,
@@ -229,8 +304,7 @@ int http_response_handle_cachable(request_st * const r, const buffer *lmod, cons
 		/* last-modified handling */
 		if (buffer_is_equal(lmod, vb)
 		    || !http_date_if_modified_since(BUF_PTR_LEN(vb), lmtime)) {
-			r->http_status = 304;
-			return HANDLER_FINISHED;
+			return http_response_304(r);
 		}
 	}
 
@@ -288,7 +362,6 @@ static void http_response_header_clear (request_st * const r) {
 
 void http_response_reset (request_st * const r) {
     r->http_status = 0;
-    r->con->is_writable = 1;
     r->handler_module = NULL;
     if (r->physical.path.ptr) { /*(skip for mod_fastcgi authorizer)*/
         buffer_clear(&r->physical.doc_root);
@@ -330,7 +403,7 @@ void http_response_send_file (request_st * const r, const buffer * const path, s
 		if (sce->fd < 0 && __builtin_expect( (0 != sce->st.st_size), 0)) {
 			r->http_status = (errno == ENOENT) ? 404 : 403;
 			if (r->conf.log_request_handling) {
-				log_perror(r->conf.errh, __FILE__, __LINE__,
+				log_pdebug(r->conf.errh, __FILE__, __LINE__,
 				  "file open failed: %s", path->ptr);
 			}
 			return;
@@ -341,9 +414,9 @@ void http_response_send_file (request_st * const r, const buffer * const path, s
 	    && 0 != stat_cache_path_contains_symlink(path, r->conf.errh)) {
 		r->http_status = 403;
 		if (r->conf.log_request_handling) {
-			log_error(r->conf.errh, __FILE__, __LINE__,
+			log_debug(r->conf.errh, __FILE__, __LINE__,
 			  "-- access denied due symlink restriction");
-			log_error(r->conf.errh, __FILE__, __LINE__,
+			log_debug(r->conf.errh, __FILE__, __LINE__,
 			  "Path         : %s", path->ptr);
 		}
 		return;
@@ -353,7 +426,7 @@ void http_response_send_file (request_st * const r, const buffer * const path, s
 	if (__builtin_expect( (!S_ISREG(sce->st.st_mode)), 0)) {
 		r->http_status = 403;
 		if (r->conf.log_file_not_found) {
-			log_error(r->conf.errh, __FILE__, __LINE__,
+			log_debug(r->conf.errh, __FILE__, __LINE__,
 			  "not a regular file: %s -> %s",
 			  r->uri.path.ptr, path->ptr);
 		}
@@ -948,19 +1021,19 @@ static int http_response_process_headers(request_st * const restrict r, http_res
             break;
           case HTTP_HEADER_UPGRADE:
             /*(technically, should also verify Connection: upgrade)*/
-            /*(flag only for mod_proxy and mod_cgi (for now))*/
-            if (opts->backend != BACKEND_PROXY && opts->backend != BACKEND_CGI)
-                continue;
-            if (r->http_version >= HTTP_VERSION_2 && !r->h2_connect_ext)
+            /* likely broken behavior if backend thinks connection is being
+             * upgraded but client does not receive Upgrade, but if policy
+             * disallowed Upgrade, strip out Upgrade response header */
+            if (!opts->upgrade || r->http_status != 101)
                 continue;
             break;
           case HTTP_HEADER_CONNECTION:
             if (opts->backend == BACKEND_PROXY) continue;
+            if (r->http_version >= HTTP_VERSION_2) continue;
             /*(simplistic attempt to honor backend request to close)*/
             if (http_header_str_contains_token(value, end - value,
                                                CONST_STR_LEN("close")))
                 r->keep_alive = 0;
-            if (r->http_version >= HTTP_VERSION_2) continue;
             break;
           case HTTP_HEADER_CONTENT_TYPE:
             if (end - value >= 22   /*(prefix match probably good enough)*/
@@ -1038,29 +1111,11 @@ static int http_response_process_headers(request_st * const restrict r, http_res
 }
 
 
-static http_response_send_1xx_cb http_response_send_1xx_h1;
-static http_response_send_1xx_cb http_response_send_1xx_h2;
-
-void
-http_response_send_1xx_cb_set (http_response_send_1xx_cb fn, int vers)
-{
-    if (vers >= HTTP_VERSION_2)
-        http_response_send_1xx_h2 = fn;
-    else if (vers == HTTP_VERSION_1_1)
-        http_response_send_1xx_h1 = fn;
-}
-
-
 int
 http_response_send_1xx (request_st * const r)
 {
-    http_response_send_1xx_cb http_response_send_1xx_fn = NULL;
-    if (r->http_version >= HTTP_VERSION_2)
-        http_response_send_1xx_fn = http_response_send_1xx_h2;
-    else if (r->http_version == HTTP_VERSION_1_1)
-        http_response_send_1xx_fn = http_response_send_1xx_h1;
-
-    if (http_response_send_1xx_fn && !http_response_send_1xx_fn(r, r->con))
+    if (    http_dispatch[r->http_version].send_1xx
+        && !http_dispatch[r->http_version].send_1xx(r, r->con))
         return 0; /* error occurred */
 
     http_response_header_clear(r);
@@ -1183,6 +1238,31 @@ handler_t http_response_parse_headers(request_st * const r, http_response_opts *
         return HANDLER_FINISHED;
     }
 
+    if (!opts->authorizer && light_btst(r->resp_htags, HTTP_HEADER_UPGRADE)) {
+        /*(already checked in http_response_process_headers())*/
+        /*if (opts->upgrade && r->http_status == 101)*/
+        {
+            /* 101 Switching Protocols; transition to transparent proxy */
+            opts->upgrade = 2;
+            if (r->h2_connect_ext) {
+                r->http_status = 200; /* OK (response status for CONNECT) */
+                http_header_response_unset(r, HTTP_HEADER_UPGRADE,
+                                           CONST_STR_LEN("Upgrade"));
+                http_header_response_unset(r, HTTP_HEADER_OTHER,
+                                         CONST_STR_LEN("Sec-WebSocket-Accept"));
+            }
+            http_response_upgrade_read_body_unknown(r);
+        }
+    }
+    else if (__builtin_expect( (r->h2_connect_ext != 0), 0)
+             && r->http_status < 300) {
+        /*(not handling other 1xx intermediate responses here; not expected)*/
+        http_response_body_clear(r, 0);
+        r->handler_module = NULL;
+        r->http_status = 405; /* Method Not Allowed */
+        return HANDLER_FINISHED;
+    }
+
     if (opts->local_redir && r->http_status >= 300 && r->http_status < 400
         && 0 == blen) {
         /* (Might not have begun to receive body yet, but do skip local-redir
@@ -1323,8 +1403,30 @@ handler_t http_response_read(request_st * const r, http_response_opts * const op
             avail = chunk_buffer_prepare_append(b, avail);
         }
 
+      #ifdef _WIN32
+        n = recv(fd, b->ptr+buffer_clen(b), avail, 0);
+        if (n == SOCKET_ERROR) {
+            switch (WSAGetLastError()) {
+              case WSAEWOULDBLOCK:
+              case WSAEINTR:
+                if (buffer_is_blank(b))
+                    chunk_buffer_yield(b); /*(improve large buf reuse)*/
+                return HANDLER_GO_ON;
+              case WSAECONNRESET:
+                if (opts->backend == BACKEND_CGI) {
+                    /* treat as EOF if WSAECONNRESET on local socket from CGI */
+                    n = 0;
+                    break;
+                }
+                __attribute_fallthrough__
+              default:
+                log_serror(r->conf.errh, __FILE__, __LINE__,
+                  "recv() %d %d", r->con->fd, fd);
+                return HANDLER_ERROR;
+            }
+        }
+      #else
         n = read(fd, b->ptr+buffer_clen(b), avail);
-
         if (n < 0) {
             switch (errno) {
               case EAGAIN:
@@ -1343,6 +1445,7 @@ handler_t http_response_read(request_st * const r, http_response_opts * const op
                 return HANDLER_ERROR;
             }
         }
+      #endif
 
         buffer_commit(b, (size_t)n);
       #ifdef __COVERITY__
