@@ -10,17 +10,11 @@
 #include "http_chunk.h"
 #include "chunk.h"
 #include "stat_cache.h"
-#include "fdevent.h"
 #include "log.h"
 #include "request.h"
 
-#include <sys/types.h>
-#include <sys/stat.h>
+#include "sys-unistd.h" /* <unistd.h> */
 
-#include <stdlib.h>
-#include <unistd.h>
-
-#include <errno.h>
 #include <string.h>
 
 __attribute_noinline__
@@ -234,7 +228,6 @@ int http_chunk_append_buffer(request_st * const r, buffer * const mem) {
 __attribute_noinline__
 int http_chunk_append_mem(request_st * const r, const char * const mem, const size_t len) {
     if (0 == len) return 0;
-    force_assert(NULL != mem);
 
     chunkqueue * const cq = &r->write_queue;
 
