@@ -23,6 +23,7 @@ SOFTWARE.
 */
 
 /*(lighttpd customization)*/
+#include "first.h"
 #ifndef XXH_HEADER_NAME
 #define XXH_HEADER_NAME "algo_xxhash.h"
 #endif
@@ -34,12 +35,15 @@ SOFTWARE.
 #endif
 
 #include <assert.h>
+#include <ctype.h>
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
+#if 0
 #ifndef _WIN32 /*(<sys/queue.h> included in "lshpack.h" immediately below)*/
 #include <sys/queue.h>
 #endif
+#endif
 
 #include "lshpack.h"
 #if LS_HPACK_EMIT_TEST_CODE
@@ -1693,6 +1697,12 @@ lshpack_dec_decode (struct lshpack_dec *dec,
 
         indexed_type = LSHPACK_NO_INDEX;
     }
+
+    if (index == LSHPACK_HDR_UNKNOWN && s == src_end)
+    {
+        return LSHPACK_ERR_BAD_DATA;
+    }
+    
     if (index != LSHPACK_HDR_UNKNOWN && index <= LSHPACK_HDR_WWW_AUTHENTICATE)
     {
         output->hpack_index = index;
@@ -1786,6 +1796,11 @@ lshpack_dec_decode (struct lshpack_dec *dec,
         }
         if (len > UINT16_MAX)
             return LSHPACK_ERR_TOO_LARGE;
+        while(len > 0 && isspace(*(name + len - 1)))
+            --len;
+        if (len == 0)
+            return LSHPACK_ERR_BAD_DATA;
+
 #if LSHPACK_DEC_CALC_HASH
         output->flags |= LSXPACK_NAME_HASH;
         output->name_hash = XXH32(name, (size_t) len, LSHPACK_XXH_SEED);
