@@ -583,14 +583,14 @@ ajp13_create_env (handler_ctx * const hctx)
         if (0 == method_byte) break;
         x[5] = method_byte;
         /* protocol */
-        const char * const proto = get_http_version_name(r->http_version);
-        n = ajp13_enc_string(x, n, proto, strlen(proto));
+        const buffer * const proto = http_version_buf(r->http_version);
+        n = ajp13_enc_string(x, n, BUF_PTR_LEN(proto));
         if (0 == n) break;
         /* req_uri */
         n = ajp13_enc_string(x, n, BUF_PTR_LEN(&r->uri.path));
         if (0 == n) break;
         /* remote_addr */
-        n = ajp13_enc_string(x, n, BUF_PTR_LEN(&r->con->dst_addr_buf));
+        n = ajp13_enc_string(x, n, BUF_PTR_LEN(r->dst_addr_buf));
         if (0 == n) break;
         /* remote_host *//*(skip DNS lookup)*/
         n = ajp13_enc_string(x, n, NULL, 0);
@@ -772,11 +772,9 @@ ajp13_recv_0(const request_st * const r, const handler_ctx * const hctx)
             && !(r->conf.stream_response_body
                  & FDEVENT_STREAM_RESPONSE_POLLRDHUP))
             return HANDLER_GO_ON;
-        log_error(r->conf.errh, __FILE__, __LINE__,
-          "unexpected end-of-file (perhaps the ajp13 process died):"
-          "pid: %d socket: %s",
-          hctx->proc->pid, hctx->proc->connection_name->ptr);
 
+        gw_backend_error_trace(hctx, r,
+          "unexpected end-of-file (perhaps the ajp13 process died)");
         return HANDLER_ERROR;
 }
 
@@ -793,7 +791,7 @@ ajp13_recv_parse_loop (request_st * const r, handler_ctx * const hctx)
             break; /* incomplete packet header + min response payload */
         char *ptr = (char *)&header;
         uint32_t pklen = 5;
-        if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh) < 0)
+        if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh, 0) < 0)
             break;
         if (pklen != 5)
             break;
@@ -878,7 +876,7 @@ ajp13_recv_parse_loop (request_st * const r, handler_ctx * const hctx)
             else if (hctx->send_content_body) {
                 ptr = (char *)&header;
                 pklen = 7;
-                if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh) < 0)
+                if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh, 0) < 0)
                     return HANDLER_GO_ON;
                 if (pklen != 7)
                     return HANDLER_GO_ON;
@@ -910,7 +908,7 @@ ajp13_recv_parse_loop (request_st * const r, handler_ctx * const hctx)
                         /*assert(3 == plen);*/
             ptr = (char *)&header;
             pklen = 7;
-            if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh) < 0)
+            if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh, 0) < 0)
                 return HANDLER_GO_ON;
             if (pklen != 7)
                 return HANDLER_GO_ON;
@@ -921,7 +919,7 @@ ajp13_recv_parse_loop (request_st * const r, handler_ctx * const hctx)
           #if 0
             ptr = (char *)&header;
             pklen = 6;
-            if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh) < 0)
+            if (chunkqueue_peek_data(hctx->rb, &ptr, &pklen, errh, 0) < 0)
                 return HANDLER_GO_ON;
             if (pklen != 6)
                 return HANDLER_GO_ON;
@@ -991,6 +989,7 @@ ajp13_check_extension (request_st * const r, void *p_d)
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_ajp13_plugin_init (plugin *p);
 int mod_ajp13_plugin_init (plugin *p)
 {
