@@ -19,11 +19,13 @@
 #include <unistd.h>
 #endif
 
+#include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "mod_auth_api.h"
 #include "sys-crypto-md.h" /* USE_LIB_CRYPTO */
+#include "sys-unistd.h" /* <unistd.h> */
 
 #include "base64.h"
 #include "ck.h"
@@ -654,6 +656,7 @@ static int mod_authn_file_crypt_cmp(const buffer * const password, const char *
                     char ntlmhex[MD4_DIGEST_LENGTH*2+1];
                     MD4_once((unsigned char *)ntlmhash, sample, pwlen);
                     li_tohex(ntlmhex,sizeof(ntlmhex),ntlmhash,sizeof(ntlmhash));
+                    ntlmhex[MD4_DIGEST_LENGTH*2] = '\0';
 
                     /*(reuse sample buffer for salt  (FYI: expect slen == 8))*/
                     memcpy(sample, "$1$", sizeof("$1$")-1);
@@ -738,6 +741,7 @@ static handler_t mod_authn_file_htpasswd_basic(request_st * const r, void *p_d,
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_authn_file_plugin_init(plugin *p);
 int mod_authn_file_plugin_init(plugin *p) {
     p->version     = LIGHTTPD_VERSION_ID;
