@@ -19,6 +19,7 @@
 #include "base.h"
 #include "log.h"
 #include "buffer.h"
+#include "chunk.h"
 #include "fdevent.h"
 #include "http_chunk.h"
 #include "http_etag.h"
@@ -62,9 +63,10 @@
  * - basic alphabetical sorting (in C locale) is done on server side
  *   in case client does not execute javascript
  *   (otherwise, response could be streamed, which is not done)
- *      (possible future dir-listing.* config option)
+ *   (disable server-side sorting with dir-listing.sort = "disable")
  * - reading entire directory into memory for sorting large directory
  *   can lead to large memory usage if many simultaneous requests occur
+ *   (disable server-side sorting with dir-listing.sort = "disable")
  */
 
 struct dirlist_cache {
@@ -75,6 +77,7 @@ struct dirlist_cache {
 typedef struct {
 	char dir_listing;
 	char json;
+	char sort;
 	char hide_dot_files;
 	char hide_readme_file;
 	char encode_readme;
@@ -125,11 +128,14 @@ typedef struct {
 	char *path_file;
 	int dfd; /*(dirfd() owned by (DIR *))*/
 	uint32_t name_max;
+	buffer *hb;
 	buffer *jb;
 	int jcomma;
 	int jfd;
 	char *jfn;
 	uint32_t jfn_len;
+	int use_xattr;
+	const array *mimetypes;
 	plugin_config conf;
   #ifdef _WIN32
 	HANDLE hFind;
@@ -142,6 +148,7 @@ typedef struct {
 static int dirlist_max_in_progress;
 
 
+__attribute_returns_nonnull__
 static handler_ctx * mod_dirlisting_handler_ctx_init (plugin_data * const p) {
     handler_ctx *hctx = ck_calloc(1, sizeof(*hctx));
   #ifdef _WIN32
@@ -171,8 +178,11 @@ static void mod_dirlisting_handler_ctx_free (handler_ctx *hctx) {
             free(ent[i]);
         free(ent);
     }
-    if (hctx->jb) {
-        chunk_buffer_release(hctx->jb);
+    if (hctx->jb || hctx->hb) {
+        if (hctx->jb)
+            chunk_buffer_release(hctx->jb);
+        else /* (hctx->hb) */
+            chunk_buffer_release(hctx->hb);
         if (-1 != hctx->jfd)
             close(hctx->jfd);
         if (hctx->jfn) {
@@ -350,6 +360,9 @@ static void mod_dirlisting_merge_config_cpv(plugin_config * const pconf, const c
         if (cpv->vtype == T_CONFIG_LOCAL)
             pconf->cache = cpv->v.v;
         break;
+      case 16:/* dir-listing.sort */
+        pconf->sort = (char)cpv->v.u;
+        break;
       default:/* should not happen */
         return;
     }
@@ -419,6 +432,9 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
      ,{ CONST_STR_LEN("dir-listing.cache"),
         T_CONFIG_ARRAY_KVANY,
         T_CONFIG_SCOPE_CONNECTION }
+     ,{ CONST_STR_LEN("dir-listing.sort"),
+        T_CONFIG_BOOL,
+        T_CONFIG_SCOPE_CONNECTION }
      ,{ NULL, 0,
         T_CONFIG_UNSET,
         T_CONFIG_SCOPE_UNSET }
@@ -495,6 +511,8 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
                 }
                 cpv->vtype = T_CONFIG_LOCAL;
                 break;
+              case 16:/* dir-listing.sort */
+                break;
               default:/* should not happen */
                 break;
             }
@@ -512,6 +530,7 @@ SETDEFAULTS_FUNC(mod_dirlisting_set_defaults) {
     p->defaults.encode_readme = 1;
     p->defaults.encode_header = 1;
     p->defaults.auto_layout = 1;
+    p->defaults.sort = 1;
 
     /* initialize p->defaults from global config context */
     if (p->nconfig > 0 && p->cvlist->v.u2[1]) {
@@ -633,8 +652,10 @@ static void http_list_directory_include_file(request_st * const r, const handler
         buffer_clear(out);
         const int fd = sce->fd;
         ssize_t rd;
+        off_t off = 0;
         char buf[8192];
-        while ((rd = read(fd, buf, sizeof(buf))) > 0) {
+        while ((rd = chunk_file_pread(fd, buf, sizeof(buf), off)) > 0) {
+            off += rd;
             buffer_append_string_encoded(out, buf, (size_t)rd, ENCODING_MINIMAL_XML);
             if (out == tb) {
                 if (0 != chunkqueue_append_mem_to_tempfile(cq,
@@ -654,6 +675,25 @@ static void http_list_directory_include_file(request_st * const r, const handler
     }
 }
 
+static void http_dirlist_link (request_st * const r, const buffer *b, const char *params, size_t plen) {
+    /*(params must be at least ">" to close Link url-reference)*/
+    buffer * const tb = r->tmp_buf;
+    buffer_clear(tb);
+    buffer_append_str3(tb, CONST_STR_LEN("<"), BUF_PTR_LEN(b), params, plen);
+    http_header_response_insert(r, HTTP_HEADER_LINK,
+                                CONST_STR_LEN("Link"),
+                                BUF_PTR_LEN(tb));
+}
+
+static void http_dirlist_auto_layout_early_hints (request_st * const r, const plugin_data * const p) {
+    if (p->conf.external_css)
+        http_dirlist_link(r, p->conf.external_css,
+          CONST_STR_LEN(">; rel=\"preload\"; as=\"style\""));
+    if (p->conf.external_js)
+        http_dirlist_link(r, p->conf.external_js,
+          CONST_STR_LEN(">; rel=\"preload\"; as=\"script\""));
+}
+
 /* portions copied from mod_status
  * modified and specialized for stable dirlist sorting by name */
 static const char js_simple_table_resort[] = \
@@ -955,6 +995,76 @@ static void http_list_directory_header(request_st * const r, const handler_ctx *
 	chunkqueue_append_buffer_commit(cq);
 }
 
+static void http_list_directory_mtime(buffer * const out, const dirls_entry_t * const ent) {
+	struct tm tm;
+  #ifdef __MINGW32__
+	buffer_append_strftime(out, "%Y-%b-%d %H:%M:%S", localtime64_r(&ent->mtime, &tm));
+  #else
+	buffer_append_strftime(out, "%Y-%b-%d %T", localtime64_r(&ent->mtime, &tm));
+  #endif
+}
+
+static void http_list_directory_ent(buffer * const out, const dirls_entry_t * const ent, const char * const name) {
+	buffer_append_string_encoded(out, name, ent->namelen, ENCODING_REL_URI_PART);
+	buffer_append_string_len(out, CONST_STR_LEN("/\">"));
+	buffer_append_string_encoded(out, name, ent->namelen, ENCODING_MINIMAL_XML);
+	buffer_append_string_len(out, CONST_STR_LEN("</a>/</td><td class=\"m\">"));
+
+	http_list_directory_mtime(out, ent);
+}
+
+__attribute_noinline__
+static void http_list_directory_dirname(buffer * const out, const dirls_entry_t * const ent, const char * const name) {
+	buffer_append_string_len(out, CONST_STR_LEN("<tr class=\"d\"><td class=\"n\"><a href=\""));
+
+	http_list_directory_ent(out, ent, name);
+
+	buffer_append_string_len(out, CONST_STR_LEN("</td><td class=\"s\">- &nbsp;</td><td class=\"t\">Directory</td></tr>\n"));
+}
+
+static void http_list_directory_filename(buffer * const out, const dirls_entry_t * const ent, const char * const name, handler_ctx * const hctx) {
+	buffer_append_string_len(out, CONST_STR_LEN("<tr><td class=\"n\"><a href=\""));
+
+	http_list_directory_ent(out, ent, name);
+
+	const buffer *content_type;
+  #if defined(HAVE_XATTR) || defined(HAVE_EXTATTR) /*(pass full path)*/
+	content_type = NULL;
+	if (hctx->use_xattr) {
+		memcpy(hctx->path_file, name, ent->namelen + 1);
+		content_type = stat_cache_mimetype_by_xattr(hctx->path);
+	}
+	if (NULL == content_type)
+  #endif
+		content_type = stat_cache_mimetype_by_ext(hctx->mimetypes, name, ent->namelen);
+	if (NULL == content_type) {
+		static const buffer octet_stream =
+		  { "application/octet-stream",
+		    sizeof("application/octet-stream"), 0 };
+		content_type = &octet_stream;
+	}
+
+	char sizebuf[sizeof("999.9K")];
+	size_t buflen =
+	  http_list_directory_sizefmt(sizebuf, sizeof(sizebuf), ent->size);
+	struct const_iovec iov[] = {
+	  { CONST_STR_LEN("</td><td class=\"s\">") }
+	 ,{ sizebuf, buflen }
+	 ,{ CONST_STR_LEN("</td><td class=\"t\">") }
+	 ,{ BUF_PTR_LEN(content_type) }
+	 ,{ CONST_STR_LEN("</td></tr>\n") }
+	};
+	buffer_append_iovec(out, iov, sizeof(iov)/sizeof(*iov));
+}
+
+static void http_list_directory_dir(buffer * const out, const dirls_entry_t * const ent) {
+	http_list_directory_dirname(out, ent, DIRLIST_ENT_NAME(ent));
+}
+
+static void http_list_directory_file(buffer * const out, const dirls_entry_t * const ent, handler_ctx * const hctx) {
+	http_list_directory_filename(out, ent, DIRLIST_ENT_NAME(ent), hctx);
+}
+
 static void http_list_directory_footer(request_st * const r, const handler_ctx * const p) {
 
 	chunkqueue * const cq = &r->write_queue;
@@ -998,6 +1108,38 @@ static void http_list_directory_footer(request_st * const r, const handler_ctx *
 	}
 }
 
+__attribute_noinline__
+static void http_list_directory_jsonname(buffer * const out, const dirls_entry_t * const ent, const char * const name, handler_ctx * const hctx, const int isdir) {
+	if (__builtin_expect( (hctx->jcomma), 1))/*(to avoid excess comma)*/
+		buffer_append_string_len(out, CONST_STR_LEN(",{\"name\":\""));
+	else {
+		hctx->jcomma = 1;
+		buffer_append_string_len(out, CONST_STR_LEN( "{\"name\":\""));
+	}
+	buffer_append_bs_escaped_json(out, name, ent->namelen);
+
+	const char *t;
+	size_t tlen;
+	if (!isdir) {
+		t =           "\",\"type\":\"file\",\"size\":";
+		tlen = sizeof("\",\"type\":\"file\",\"size\":")-1;
+	}
+	else {
+		t =           "\",\"type\":\"dir\",\"size\":";
+		tlen = sizeof("\",\"type\":\"dir\",\"size\":")-1;
+	}
+	char sstr[LI_ITOSTRING_LENGTH];
+	char mstr[LI_ITOSTRING_LENGTH];
+	struct const_iovec iov[] = {
+	  { t, tlen }
+	 ,{ sstr, li_itostrn(sstr, sizeof(sstr), ent->size) }
+	 ,{ CONST_STR_LEN(",\"mtime\":") }
+	 ,{ mstr, li_itostrn(mstr, sizeof(mstr), ent->mtime) }
+	 ,{ CONST_STR_LEN("}") }
+	};
+	buffer_append_iovec(out, iov, sizeof(iov)/sizeof(*iov));
+}
+
 static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
     const uint32_t dlen = buffer_clen(&r->physical.path);
 #ifdef _WIN32
@@ -1065,8 +1207,6 @@ static int http_open_directory(request_st * const r, handler_ctx * const hctx) {
 }
 
 static int http_read_directory(handler_ctx * const p) {
-	const int hide_dotfiles = p->conf.hide_dot_files;
-	const uint32_t name_max = p->name_max;
   #ifdef _WIN32
 	int count = 0;
 	if (INVALID_HANDLE_VALUE == p->hFind) {
@@ -1095,7 +1235,7 @@ static int http_read_directory(handler_ctx * const p) {
 		const uint32_t dsz = (uint32_t) _D_EXACT_NAMLEN(dent);
 	  #endif
 		if (d_name[0] == '.') {
-			if (hide_dotfiles)
+			if (p->conf.hide_dot_files)
 				continue;
 			if (d_name[1] == '\0')
 				continue;
@@ -1122,7 +1262,7 @@ static int http_read_directory(handler_ctx * const p) {
 		/* NOTE: the manual says, d_name is never more than NAME_MAX
 		 *       so this should actually not be a buffer-overflow-risk
 		 */
-		if (dsz > name_max) continue;
+		if (dsz > p->name_max) continue;
 	  #ifdef __COVERITY__
 		/* For some reason, Coverity overlooks the strlen() performed
 		 * a few lines above and thinks memcpy() below might access
@@ -1142,81 +1282,45 @@ static int http_read_directory(handler_ctx * const p) {
 	  #endif
 	  #endif
 
-		if (p->jb) { /* json output */
-			if (__builtin_expect( (p->jcomma), 1))/*(to avoid excess comma)*/
-				buffer_append_string_len(p->jb, CONST_STR_LEN(",{\"name\":\""));
-			else {
-				p->jcomma = 1;
-				buffer_append_string_len(p->jb, CONST_STR_LEN( "{\"name\":\""));
-			}
-			buffer_append_bs_escaped_json(p->jb, d_name, dsz);
+		/* const dirls_entry_t ent = { namelen, mtime, size } */
+	  #ifdef _WIN32 /*(convert 100ns ticks since 1 Jan 1601 to unix time_t)*/
+		/*(future: preserve FILETIME here and use Windows fn to format time
+		 * below instead of localtime_r() and buffer_append_strftime())*/
+		const dirls_entry_t ent = {
+		  dsz,
+		  (unix_time64_t)
+		    ((((int64_t)p->ffd.ftLastWriteTime.dwHighDateTime << 32)
+		              | p->ffd.ftLastWriteTime.dwLowDateTime)
+		    / 10000000 - 11644473600LL),
+		  ((int64_t)p->ffd.nFileSizeHigh << 32) | p->ffd.nFileSizeLow
+		};
+		const int isdir = (p->ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY);
+	  #else
+		const dirls_entry_t ent = { dsz, TIME64_CAST(st.st_mtime), st.st_size };
+		const int isdir = S_ISDIR(st.st_mode);
+	  #endif
 
-			const char *t;
-			size_t tlen;
-		  #ifndef _WIN32
-			if (!S_ISDIR(st.st_mode))
-		  #else
-			if (!(p->ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
-		  #endif
-			{
-				t =           "\",\"type\":\"file\",\"size\":";
-				tlen = sizeof("\",\"type\":\"file\",\"size\":")-1;
-			}
-			else {
-				t =           "\",\"type\":\"dir\",\"size\":";
-				tlen = sizeof("\",\"type\":\"dir\",\"size\":")-1;
-			}
-			char sstr[LI_ITOSTRING_LENGTH];
-			char mstr[LI_ITOSTRING_LENGTH];
-			struct const_iovec iov[] = {
-			  { t, tlen }
-			 #ifndef _WIN32
-			 ,{ sstr, li_itostrn(sstr, sizeof(sstr), st.st_size) }
-			 ,{ CONST_STR_LEN(",\"mtime\":") }
-			 ,{ mstr, li_itostrn(mstr, sizeof(mstr), TIME64_CAST(st.st_mtime)) }
-			 #else
-			 ,{ sstr, li_itostrn(sstr, sizeof(sstr),
-			                     ((int64_t)p->ffd.nFileSizeHigh << 32)
-			                             | p->ffd.nFileSizeLow) }
-			 ,{ CONST_STR_LEN(",\"mtime\":") }
-			 ,{ mstr, li_itostrn(mstr, sizeof(mstr),
-			            ((((int64_t)p->ffd.ftLastWriteTime.dwHighDateTime << 32)
-			                      | p->ffd.ftLastWriteTime.dwLowDateTime)
-			            / 10000000 - 11644473600LL)) }
-			 #endif
-			 ,{ CONST_STR_LEN("}") }
-			};
-			buffer_append_iovec(p->jb, iov, sizeof(iov)/sizeof(*iov));
+		if (p->jb) { /* json output */
+			http_list_directory_jsonname(p->jb, &ent, d_name, p, isdir);
 			continue;
 		}
 
-	  #ifndef _WIN32
-		dirls_list_t * const list = !S_ISDIR(st.st_mode) ? &p->files : &p->dirs;
-	  #else  /* _WIN32 */
-		dirls_list_t * const list =
-		  !(p->ffd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
-		    ? &p->files
-		    : &p->dirs;
-	  #endif /* _WIN32 */
+		if (p->hb) { /* html output **unsorted** */
+			if (isdir)
+				http_list_directory_dirname(p->hb, &ent, d_name);
+			else
+				http_list_directory_filename(p->hb, &ent, d_name, p);
+			continue;
+		}
+
+		dirls_list_t * const list = !isdir ? &p->files : &p->dirs;
 		if (!(list->used & (DIRLIST_BLOB_SIZE-1)))
 			ck_realloc_u32((void **)&list->ent, list->used,
 			               DIRLIST_BLOB_SIZE, sizeof(*list->ent));
 		dirls_entry_t * const tmp = list->ent[list->used++] =
-		  (dirls_entry_t*) ck_malloc(sizeof(dirls_entry_t) + 1 + dsz);
-	  #ifdef _WIN32 /*(convert 100ns ticks since 1 Jan 1601 to unix time_t)*/
-		/*(future: preserve FILETIME here and use Windows fn to format time
-		 * below instead of localtime_r() and buffer_append_strftime())*/
-		tmp->mtime = (time_t)
-                  ((((int64_t)p->ffd.ftLastWriteTime.dwHighDateTime << 32)
-		            | p->ffd.ftLastWriteTime.dwLowDateTime)
-                  / 10000000 - 11644473600LL);
-		tmp->size = ((int64_t)p->ffd.nFileSizeHigh << 32) | p->ffd.nFileSizeLow;
-	  #else
-		tmp->mtime = st.st_mtime;
-		tmp->size  = st.st_size;
-	  #endif
-		tmp->namelen = dsz;
-		memcpy(DIRLIST_ENT_NAME(tmp), d_name, dsz + 1);
+		  (dirls_entry_t*) ck_malloc(sizeof(dirls_entry_t) + 1 + ent.namelen);
+		*tmp = ent; /* copy struct */
+		memcpy(DIRLIST_ENT_NAME(tmp), d_name, ent.namelen + 1);
 	}
   #ifdef _WIN32
 	  while (++count < DIRLIST_BATCH && FindNextFileW(p->hFind, &p->ffd) != 0);
@@ -1244,9 +1348,6 @@ static void http_list_directory(request_st * const r, handler_ctx * const hctx)
 	if (dirs->used) http_dirls_sort(dirs->ent, dirs->used);
 	if (files->used) http_dirls_sort(files->ent, files->used);
 
-	char sizebuf[sizeof("999.9K")];
-	struct tm tm;
-
 	/* generate large directory listings into tempfiles
 	 * (estimate approx 200-256 bytes of HTML per item; could be up to ~512) */
 	chunkqueue * const cq = &r->write_queue;
@@ -1260,20 +1361,7 @@ static void http_list_directory(request_st * const r, handler_ctx * const hctx)
 	/* directories */
 	dirls_entry_t ** const dirs_ent = dirs->ent;
 	for (uint32_t i = 0, used = dirs->used; i < used; ++i) {
-		dirls_entry_t * const tmp = dirs_ent[i];
-
-		buffer_append_string_len(out, CONST_STR_LEN("<tr class=\"d\"><td class=\"n\"><a href=\""));
-		buffer_append_string_encoded(out, DIRLIST_ENT_NAME(tmp), tmp->namelen, ENCODING_REL_URI_PART);
-		buffer_append_string_len(out, CONST_STR_LEN("/\">"));
-		buffer_append_string_encoded(out, DIRLIST_ENT_NAME(tmp), tmp->namelen, ENCODING_MINIMAL_XML);
-		buffer_append_string_len(out, CONST_STR_LEN("</a>/</td><td class=\"m\">"));
-	  #ifdef __MINGW32__
-		buffer_append_strftime(out, "%Y-%b-%d %H:%M:%S", localtime64_r(&tmp->mtime, &tm));
-	  #else
-		buffer_append_strftime(out, "%Y-%b-%d %T", localtime64_r(&tmp->mtime, &tm));
-	  #endif
-		buffer_append_string_len(out, CONST_STR_LEN("</td><td class=\"s\">- &nbsp;</td><td class=\"t\">Directory</td></tr>\n"));
-
+		http_list_directory_dir(out, dirs_ent[i]);
 		if (buffer_string_space(out) < 256) {
 			if (out == tb) {
 				if (0 != chunkqueue_append_mem_to_tempfile(cq,
@@ -1286,48 +1374,9 @@ static void http_list_directory(request_st * const r, handler_ctx * const hctx)
 	}
 
 	/* files */
-	const array * const mimetypes = r->conf.mimetypes;
 	dirls_entry_t ** const files_ent = files->ent;
 	for (uint32_t i = 0, used = files->used; i < used; ++i) {
-		dirls_entry_t * const tmp = files_ent[i];
-		const buffer *content_type;
-	  #if defined(HAVE_XATTR) || defined(HAVE_EXTATTR) /*(pass full path)*/
-		content_type = NULL;
-		if (r->conf.use_xattr) {
-			memcpy(hctx->path_file, DIRLIST_ENT_NAME(tmp), tmp->namelen + 1);
-			content_type = stat_cache_mimetype_by_xattr(hctx->path);
-		}
-		if (NULL == content_type)
-	  #endif
-			content_type = stat_cache_mimetype_by_ext(mimetypes, DIRLIST_ENT_NAME(tmp), tmp->namelen);
-		if (NULL == content_type) {
-			static const buffer octet_stream =
-			  { "application/octet-stream",
-			    sizeof("application/octet-stream"), 0 };
-			content_type = &octet_stream;
-		}
-
-		buffer_append_string_len(out, CONST_STR_LEN("<tr><td class=\"n\"><a href=\""));
-		buffer_append_string_encoded(out, DIRLIST_ENT_NAME(tmp), tmp->namelen, ENCODING_REL_URI_PART);
-		buffer_append_string_len(out, CONST_STR_LEN("\">"));
-		buffer_append_string_encoded(out, DIRLIST_ENT_NAME(tmp), tmp->namelen, ENCODING_MINIMAL_XML);
-		buffer_append_string_len(out, CONST_STR_LEN("</a></td><td class=\"m\">"));
-	  #ifdef __MINGW32__
-		buffer_append_strftime(out, "%Y-%b-%d %H:%M:%S", localtime64_r(&tmp->mtime, &tm));
-	  #else
-		buffer_append_strftime(out, "%Y-%b-%d %T", localtime64_r(&tmp->mtime, &tm));
-	  #endif
-		size_t buflen =
-		  http_list_directory_sizefmt(sizebuf, sizeof(sizebuf), tmp->size);
-		struct const_iovec iov[] = {
-		  { CONST_STR_LEN("</td><td class=\"s\">") }
-		 ,{ sizebuf, buflen }
-		 ,{ CONST_STR_LEN("</td><td class=\"t\">") }
-		 ,{ BUF_PTR_LEN(content_type) }
-		 ,{ CONST_STR_LEN("</td></tr>\n") }
-		};
-		buffer_append_iovec(out, iov, sizeof(iov)/sizeof(*iov));
-
+		http_list_directory_file(out, files_ent[i], hctx);
 		if (buffer_string_space(out) < 256) {
 			if (out == tb) {
 				if (0 != chunkqueue_append_mem_to_tempfile(cq,
@@ -1370,16 +1419,12 @@ static void mod_dirlisting_response (request_st * const r, handler_ctx * const h
     http_list_directory(r, hctx);
     http_list_directory_footer(r, hctx);
     mod_dirlisting_content_type(r, hctx->conf.encoding);
-
-    r->resp_body_finished = 1;
 }
 
 
-static void mod_dirlisting_json_append (request_st * const r, handler_ctx * const hctx, const int fin) {
-    buffer * const jb = hctx->jb;
-    if (fin)
-        buffer_append_string_len(jb, CONST_STR_LEN("]}"));
-    else if (buffer_clen(jb) < 16384-1024)
+static void mod_dirlisting_stream_append (request_st * const r, handler_ctx * const hctx, const int fin) {
+    buffer * const jb = hctx->jb ? hctx->jb : hctx->hb;
+    if (!fin && buffer_clen(jb) < 16384-1024)
         return; /* aggregate bunches of entries, even if streaming response */
 
     if (hctx->jfn) {
@@ -1397,9 +1442,7 @@ static void mod_dirlisting_json_append (request_st * const r, handler_ctx * cons
          * files, it is expected that only very large directories will spill to
          * temporary files, and even then most responses will be less than 1 MB.
          * The cache path can be different from server.upload-dirs.
-         * Note: since responses are not expected to be large, no effort is
-         * currently made here to handle FDEVENT_STREAM_RESPONSE_BUFMIN and to
-         * defer reading more from directory while data is sent to client */
+         */
     }
 
     http_chunk_append_buffer(r, jb); /* clears jb */
@@ -1412,9 +1455,11 @@ static handler_t mod_dirlisting_cache_check (request_st * const r, plugin_data *
 __attribute_noinline__
 static void mod_dirlisting_cache_add (request_st * const r, handler_ctx * const hctx);
 __attribute_noinline__
-static void mod_dirlisting_cache_json_init (request_st * const r, handler_ctx * const hctx);
+static void mod_dirlisting_cache_stream_add_footer (request_st * const r, handler_ctx * const hctx);
 __attribute_noinline__
-static void mod_dirlisting_cache_json (request_st * const r, handler_ctx * const hctx);
+static void mod_dirlisting_cache_stream_init (request_st * const r, handler_ctx * const hctx);
+__attribute_noinline__
+static void mod_dirlisting_cache_stream (request_st * const r, handler_ctx * const hctx);
 
 
 URIHANDLER_FUNC(mod_dirlisting_subrequest_start) {
@@ -1431,9 +1476,9 @@ URIHANDLER_FUNC(mod_dirlisting_subrequest_start) {
 	if (!p->conf.dir_listing) return HANDLER_GO_ON;
 
 	if (r->conf.log_request_handling) {
-		log_error(r->conf.errh, __FILE__, __LINE__,
+		log_debug(r->conf.errh, __FILE__, __LINE__,
 		  "-- handling the request as Dir-Listing");
-		log_error(r->conf.errh, __FILE__, __LINE__,
+		log_debug(r->conf.errh, __FILE__, __LINE__,
 		  "URI          : %s", r->uri.path.ptr);
 	}
 
@@ -1450,11 +1495,29 @@ URIHANDLER_FUNC(mod_dirlisting_subrequest_start) {
 	}
   #endif
 
-	/* TODO: add option/mechanism to enable json output */
   #if 0 /* XXX: ??? might this be enabled accidentally by clients ??? */
+	/* XXX: would have to add "Vary: Accept" response header, too */
 	const buffer * const vb =
 	  http_header_request_get(r, HTTP_HEADER_ACCEPT, CONST_STR_LEN("Accept"));
 	p->conf.json = (vb && strstr(vb->ptr, "application/json")); /*(coarse)*/
+	if (p->conf.json) p->conf.auto_layout = 0;
+  #else
+	/* check URL for /<path>/?json to enable json output */
+	if (buffer_clen(&r->uri.query) == sizeof("json")-1
+	    && 0 == memcmp(r->uri.query.ptr, CONST_STR_LEN("json")-1)) {
+	  #if 0
+		/* streaming response not set here for mod_deflate (which
+		 * currently does not compress incomplete streaming responses),
+		 * since json response is generally highly compressible.
+		 * Admin should enable streaming response in lighttpd.conf,
+		 * if desired. */
+		if (!(r->conf.stream_response_body
+		      & (FDEVENT_STREAM_RESPONSE|FDEVENT_STREAM_RESPONSE_BUFMIN)))
+			r->conf.stream_response_body |= FDEVENT_STREAM_RESPONSE;
+	  #endif
+		p->conf.json = 1;
+		p->conf.auto_layout = 0;
+	}
   #endif
 
 	if (p->conf.cache) {
@@ -1476,6 +1539,8 @@ URIHANDLER_FUNC(mod_dirlisting_subrequest_start) {
 	}
 
 	handler_ctx * const hctx = mod_dirlisting_handler_ctx_init(p);
+	hctx->use_xattr = r->conf.use_xattr;
+	hctx->mimetypes = r->conf.mimetypes;
 
 	/* future: might implement a queue to limit max number of dirlisting
 	 * requests being serviced in parallel (increasing disk I/O), and if
@@ -1499,25 +1564,62 @@ URIHANDLER_FUNC(mod_dirlisting_subrequest_start) {
 	++p->processing;
 
 	if (p->conf.json) {
-		hctx->jfd = -1;
 		hctx->jb = chunk_buffer_acquire();
 		buffer_append_string_len(hctx->jb, CONST_STR_LEN("{["));
-		if (p->conf.cache)
-			mod_dirlisting_cache_json_init(r, hctx);
 		http_header_response_set(r, HTTP_HEADER_CONTENT_TYPE,
 		                         CONST_STR_LEN("Content-Type"),
 		                         CONST_STR_LEN("application/json"));
+	}
+	else {
+		if (p->conf.auto_layout)
+			http_dirlist_auto_layout_early_hints(r, p);
+		if (!p->conf.sort) {
+			mod_dirlisting_content_type(r, hctx->conf.encoding);
+			http_list_directory_header(r, hctx);
+			hctx->hb = chunk_buffer_acquire();
+		}
+	}
+
+	if (hctx->jb || hctx->hb) {
+		hctx->jfd = -1;
+		if (p->conf.cache)
+			mod_dirlisting_cache_stream_init(r, hctx);
 		r->http_status = 200;
 		r->resp_body_started = 1;
 	}
 
 	r->plugin_ctx[p->id] = hctx;
 	r->handler_module = p->self;
-	return mod_dirlisting_subrequest(r, p);
+	handler_t rc = mod_dirlisting_subrequest(r, p);
+
+	if (rc == HANDLER_WAIT_FOR_EVENT && p->conf.auto_layout
+	    && (p->conf.external_js || p->conf.external_css)
+              /*(skip if might stream unsorted since r->http_status and
+               * Content-Type would have to be saved/restored for response,
+               * as well as any partial response body of html dir header)*/
+	    && 0 == r->resp_body_started
+	      /* (paranoia: do not send 103 for HTTP/1.x; only for HTTP/2 +)
+	       *  https://www.rfc-editor.org/rfc/rfc8297.html#section-3 */
+	    && r->http_version >= HTTP_VERSION_2) {
+		/* send 103 Early Hints intermediate response (send once only)*/
+		r->http_status = 103; /* 103 Early Hints */
+		if (!http_response_send_1xx(r))
+			rc = HANDLER_ERROR;
+	}
+
+	return rc;
 }
 
 
 SUBREQUEST_FUNC(mod_dirlisting_subrequest) {
+    if ((r->conf.stream_response_body & FDEVENT_STREAM_RESPONSE_BUFMIN)
+        && chunkqueue_length(&r->write_queue) > 65536 - 4096
+        && !r->con->is_writable)
+        /* defer reading more from directory while data is sent to client
+         * (must check !r->con->is_writable or else r may not be rescheduled to
+         *  run and produce more output since r->write_queue sent out later) */
+        return HANDLER_WAIT_FOR_EVENT;
+
     plugin_data * const p = p_d;
     handler_ctx * const hctx = r->plugin_ctx[p->id];
     if (NULL == hctx) return HANDLER_GO_ON; /*(should not happen)*/
@@ -1525,22 +1627,26 @@ SUBREQUEST_FUNC(mod_dirlisting_subrequest) {
     handler_t rc = http_read_directory(hctx);
     switch (rc) {
       case HANDLER_FINISHED:
-        if (hctx->jb) { /* (hctx->conf.json) */
-            mod_dirlisting_json_append(r, hctx, 1);
-            r->resp_body_finished = 1;
+        if (hctx->jb || hctx->hb) { /* (hctx->conf.json || !hctx->conf.sort) */
+            if (hctx->jb)
+                buffer_append_string_len(hctx->jb, CONST_STR_LEN("]}"));
+            mod_dirlisting_stream_append(r, hctx, 1);
+            if (hctx->hb)
+                mod_dirlisting_cache_stream_add_footer(r, hctx);
             if (hctx->jfn) /* (also (hctx->conf.cache) */
-                mod_dirlisting_cache_json(r, hctx);
+                mod_dirlisting_cache_stream(r, hctx);
         }
         else {
             mod_dirlisting_response(r, hctx);
             if (hctx->conf.cache)
                 mod_dirlisting_cache_add(r, hctx);
         }
+        r->resp_body_finished = 1;
         mod_dirlisting_reset(r, p); /*(release resources, including hctx)*/
         break;
       case HANDLER_WAIT_FOR_EVENT: /*(used here to mean 'yield')*/
-        if (hctx->jb)   /* (hctx->conf.json) */
-            mod_dirlisting_json_append(r, hctx, 0);
+        if (hctx->jb || hctx->hb)  /* (hctx->conf.json || !hctx->conf.sort) */
+            mod_dirlisting_stream_append(r, hctx, 0);
         joblist_append(r->con);
         break;
       default:
@@ -1573,6 +1679,20 @@ static void mod_dirlisting_cache_control (request_st * const r, unix_time64_t ma
 }
 
 
+static void mod_dirlisting_cache_etag (request_st * const r, int fd)
+{
+    if (0 != r->conf.etag_flags) {
+        struct stat st;
+        if (0 == fstat(fd, &st)) {
+            buffer * const vb =
+              http_header_response_set_ptr(r, HTTP_HEADER_ETAG,
+                                           CONST_STR_LEN("ETag"));
+            http_etag_create(vb, &st, r->conf.etag_flags);
+        }
+    }
+}
+
+
 static handler_t mod_dirlisting_cache_check (request_st * const r, plugin_data * const p) {
     /* optional: an external process can trigger a refresh by deleting the cache
      * entry when the external process detects (or initiates) changes to dir */
@@ -1627,6 +1747,8 @@ static handler_t mod_dirlisting_cache_check (request_st * const r, plugin_data *
                                      CONST_STR_LEN("ETag"),
                                      BUF_PTR_LEN(etag));
     }
+    if (p->conf.auto_layout)
+        http_dirlist_auto_layout_early_hints(r, p);
 
     r->resp_body_finished = 1;
     return HANDLER_FINISHED;
@@ -1701,38 +1823,66 @@ static void mod_dirlisting_cache_add (request_st * const r, handler_ctx * const
     const int fd = fdevent_mkostemp(oldpath, 0);
     if (fd < 0) return;
     int rc = mod_dirlisting_write_cq(fd, &r->write_queue, r->conf.errh);
-  #ifdef _WIN32
-    close(fd); /*(rename fails if file is open; MS filesystem limitation)*/
-  #endif
+    if (rc)
+        mod_dirlisting_cache_etag(r, fd);
+    close(fd); /*(_WIN32 rename fails if file open; MS filesystem limitation)*/
     if (rc && 0 == fdevent_rename(oldpath, newpath)) {
         stat_cache_invalidate_entry(newpath, len);
-        /* Cache-Control and ETag (also done in mod_dirlisting_cache_check())*/
         mod_dirlisting_cache_control(r, hctx->conf.cache->max_age);
-        if (0 != r->conf.etag_flags) {
-            struct stat st;
-          #ifdef _WIN32
-            if (0 == stat(newpath, &st))
-          #else
-            if (0 == fstat(fd, &st))
-          #endif
-            {
-                buffer * const vb =
-                  http_header_response_set_ptr(r, HTTP_HEADER_ETAG,
-                                               CONST_STR_LEN("ETag"));
-                http_etag_create(vb, &st, r->conf.etag_flags);
-            }
-        }
     }
     else
         unlink(oldpath);
-  #ifndef _WIN32
-    close(fd);
-  #endif
 }
 
 
 __attribute_noinline__
-static void mod_dirlisting_cache_json_init (request_st * const r, handler_ctx * const hctx) {
+static void mod_dirlisting_cache_stream_append_cq (request_st * const r, handler_ctx * const hctx) {
+    if (chunkqueue_is_empty(&r->write_queue) || hctx->jfd == -1) return;
+
+    /* append HTML in r->write_queue to cache file
+     * (but must abort cache file if streaming response and sending HTTP/1.1
+     *  chunked response due to http_list_directory_footer() writing directly
+     *  to r->write_queue, which will be written in HTTP chunked format) */
+    if (r->resp_send_chunked
+        || !mod_dirlisting_write_cq(hctx->jfd, &r->write_queue, r->conf.errh)) {
+        close(hctx->jfd);
+        hctx->jfd = -1;
+        unlink(hctx->jfn);
+        free(hctx->jfn);
+        hctx->jfn = NULL;
+    }
+}
+
+
+static void mod_dirlisting_cache_stream_add_header (request_st * const r, handler_ctx * const hctx) {
+    mod_dirlisting_cache_stream_append_cq(r, hctx);
+}
+
+
+__attribute_noinline__
+static void mod_dirlisting_cache_stream_add_footer (request_st * const r, handler_ctx * const hctx) {
+    /* save and restore r->write_queue around adding footer and updating cache
+     * (due to http_list_directory_footer() appending to r->write_queue)*/
+    chunkqueue * const cq = &r->write_queue;
+    chunkqueue in;
+    memset(&in, 0, sizeof(in));
+    chunkqueue_append_chunkqueue(&in, cq);
+    cq->bytes_in  -= in.bytes_in;
+    cq->bytes_out -= in.bytes_in;
+
+    http_list_directory_footer(r, hctx);
+    mod_dirlisting_cache_stream_append_cq(r, hctx);
+
+    off_t len = chunkqueue_length(cq);
+    chunkqueue_append_chunkqueue(&in, cq);
+    cq->bytes_in  -= len;
+    cq->bytes_out -= len;
+    chunkqueue_append_chunkqueue(cq, &in);
+}
+
+
+__attribute_noinline__
+static void mod_dirlisting_cache_stream_init (request_st * const r, handler_ctx * const hctx) {
   #ifndef PATH_MAX
   #define PATH_MAX 4096
   #endif
@@ -1742,18 +1892,23 @@ static void mod_dirlisting_cache_json_init (request_st * const r, handler_ctx *
     if (!stat_cache_path_isdir(tb)
         && 0 != mkdir_recursive(tb->ptr, buffer_clen(hctx->conf.cache->path)))
         return;
-    buffer_append_string_len(tb, CONST_STR_LEN("dirlist.json.XXXXXX"));
+    buffer_append_string_len(tb, hctx->jb
+                                  ? "dirlist.json.XXXXXX"
+                                  : "dirlist.html.XXXXXX",
+                             sizeof("dirlist.json.XXXXXX")-1);
     const int fd = fdevent_mkostemp(tb->ptr, 0);
     if (fd < 0) return;
     hctx->jfn_len = buffer_clen(tb);
     hctx->jfd = fd;
     hctx->jfn = ck_malloc(hctx->jfn_len+1);
     memcpy(hctx->jfn, tb->ptr, hctx->jfn_len+1); /*(include '\0')*/
+    if (hctx->hb)
+        mod_dirlisting_cache_stream_add_header(r, hctx);
 }
 
 
 __attribute_noinline__
-static void mod_dirlisting_cache_json (request_st * const r, handler_ctx * const hctx) {
+static void mod_dirlisting_cache_stream (request_st * const r, handler_ctx * const hctx) {
   #ifndef PATH_MAX
   #define PATH_MAX 4096
   #endif
@@ -1763,10 +1918,15 @@ static void mod_dirlisting_cache_json (request_st * const r, handler_ctx * const
     force_assert(len < PATH_MAX);
     memcpy(newpath, hctx->jfn, len);
     newpath[len] = '\0';
-    close(hctx->jfd);
+    if (0 == r->resp_header_len) /*(response headers not yet sent)*/
+        mod_dirlisting_cache_etag(r, hctx->jfd);
+    close(hctx->jfd); /*(_WIN32 rename fails if file open; MS fs limitation)*/
     hctx->jfd = -1;
-    if (0 == fdevent_rename(hctx->jfn, newpath))
+    if (0 == fdevent_rename(hctx->jfn, newpath)) {
         stat_cache_invalidate_entry(newpath, len);
+        if (0 == r->resp_header_len) /*(response headers not yet sent)*/
+            mod_dirlisting_cache_control(r, hctx->conf.cache->max_age);
+    }
     else
         unlink(hctx->jfn);
     free(hctx->jfn);
