@@ -125,6 +125,7 @@ static void mod_proxy_merge_config_cpv(plugin_config * const pconf, const config
       case 5: /* proxy.header */
         /*if (cpv->vtype == T_CONFIG_LOCAL)*//*always true here for this param*/
         pconf->header = *(http_header_remap_opts *)cpv->v.v; /*(copies struct)*/
+        pconf->gw.upgrade = pconf->header.upgrade;
         break;
       case 6: /* proxy.replace-http-host */
         pconf->replace_http_host = cpv->v.u;
@@ -402,7 +403,7 @@ static const buffer * http_header_remap_host_match (buffer *b, size_t off, http_
                  * (If no Host in client request, then matching against empty
                  *  string will probably not match, and no remap will be
                  *  performed) */
-                k = is_req
+                k = is_req || NULL == remap_hdrs->forwarded_host
                   ? remap_hdrs->http_host
                   : remap_hdrs->forwarded_host;
                 if (NULL == k) continue;
@@ -689,7 +690,6 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
     }
 
     if (flags & PROXY_FORWARDED_BY) {
-        int family = sock_addr_get_family(&con->srv_socket->addr);
         /* Note: getsockname() and inet_ntop() are expensive operations.
          * (recommendation: do not to enable by=... unless required)
          * future: might use con->srv_socket->srv_token if addr is not
@@ -700,6 +700,7 @@ static void proxy_set_Forwarded(connection * const con, request_st * const r, co
         buffer_append_string_len(b, CONST_STR_LEN("by=\""));
       #ifdef HAVE_SYS_UN_H
         /* special-case: might need to encode unix domain socket path */
+        int family = sock_addr_get_family(&con->srv_socket->addr);
         if (family == AF_UNIX) {
             buffer_append_string_backslash_escaped(
               b, BUF_PTR_LEN(con->srv_socket->srv_token));
@@ -893,13 +894,17 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 	}
 	else if (r->h2_connect_ext) {
 	}
-	else if (-1 == r->reqbody_length
+	else if (r->reqbody_length < 0
 	         && (r->conf.stream_request_body
 	             & (FDEVENT_STREAM_REQUEST | FDEVENT_STREAM_REQUEST_BUFMIN))) {
-		if (__builtin_expect( (hctx->conf.header.force_http10), 0))
-			return http_response_reqbody_read_error(r, 411);
-		hctx->gw.stdin_append = proxy_stdin_append; /* stream chunked body */
-		buffer_append_string_len(b, CONST_STR_LEN("\r\nTransfer-Encoding: chunked"));
+		if (__builtin_expect( (hctx->conf.header.force_http10), 0)) {
+			if (-1 == r->reqbody_length)
+				return http_response_reqbody_read_error(r, 411);
+		}
+		else {
+			hctx->gw.stdin_append = proxy_stdin_append; /* stream chunked body */
+			buffer_append_string_len(b, CONST_STR_LEN("\r\nTransfer-Encoding: chunked"));
+		}
 	}
 
 	/* "Forwarded" and legacy X- headers */
@@ -933,10 +938,8 @@ static handler_t proxy_create_env(gw_handler_ctx *gwhctx) {
 			te = &ds->value; /*("trailers")*/
 			break;
 		case HTTP_HEADER_UPGRADE:
-			if (hctx->conf.header.force_http10
-			    || (r->http_version != HTTP_VERSION_1_1 && !r->h2_connect_ext))
-				continue;
-			if (!hctx->conf.header.upgrade) continue;
+			/*(already unset in gw_check_extension if not allowed)*/
+			if (hctx->conf.header.force_http10) continue;
 			if (!buffer_is_blank(&ds->value)) upgrade = &ds->value;
 			break;
 		case HTTP_HEADER_CONNECTION:
@@ -1056,38 +1059,9 @@ static handler_t proxy_response_headers(request_st * const r, struct http_respon
     handler_ctx *hctx = (handler_ctx *)opts->pdata;
     http_header_remap_opts * const remap_hdrs = &hctx->conf.header;
 
-    if (light_btst(r->resp_htags, HTTP_HEADER_UPGRADE)) {
-        if (remap_hdrs->upgrade && r->http_status == 101) {
-            /* 101 Switching Protocols; transition to transparent proxy */
-            if (r->h2_connect_ext) {
-                r->http_status = 200; /* OK (response status for CONNECT) */
-                http_header_response_unset(r, HTTP_HEADER_UPGRADE,
-                                           CONST_STR_LEN("Upgrade"));
-                http_header_response_unset(r, HTTP_HEADER_OTHER,
-                                         CONST_STR_LEN("Sec-WebSocket-Accept"));
-            }
-            gw_set_transparent(&hctx->gw);
-            http_response_upgrade_read_body_unknown(r);
-        }
-        else {
-            light_bclr(r->resp_htags, HTTP_HEADER_UPGRADE);
-          #if 0
-            /* preserve prior questionable behavior; likely broken behavior
-             * anyway if backend thinks connection is being upgraded but client
-             * does not receive Connection: upgrade */
-            http_header_response_unset(r, HTTP_HEADER_UPGRADE,
-                                       CONST_STR_LEN("Upgrade"))
-          #endif
-        }
-    }
-    else if (__builtin_expect( (r->h2_connect_ext != 0), 0)
-             && r->http_status < 300) {
-        /*(not handling other 1xx intermediate responses here; not expected)*/
-        http_response_body_clear(r, 0);
-        r->handler_module = NULL;
-        r->http_status = 405; /* Method Not Allowed */
-        return HANDLER_FINISHED;
-    }
+    /*(see gw_response_headers_upgrade())*/
+    if (opts->upgrade == 2)
+        gw_set_transparent(&hctx->gw);
 
     /* rewrite paths, if needed */
 
@@ -1136,8 +1110,6 @@ static handler_t mod_proxy_check_extension(request_st * const r, void *p_d) {
 
 		hctx->conf = p->conf; /*(copies struct)*/
 		hctx->conf.header.http_host = r->http_host;
-		hctx->conf.header.upgrade  &=
-                  (r->http_version == HTTP_VERSION_1_1 || r->h2_connect_ext);
 		/* mod_proxy currently sends all backend requests as http.
 		 * https-remap is a flag since it might not be needed if backend
 		 * honors Forwarded or X-Forwarded-Proto headers, e.g. by using
@@ -1172,6 +1144,7 @@ static handler_t mod_proxy_check_extension(request_st * const r, void *p_d) {
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_proxy_plugin_init(plugin *p);
 int mod_proxy_plugin_init(plugin *p) {
 	p->version      = LIGHTTPD_VERSION_ID;
