@@ -102,7 +102,7 @@ static handler_t
 mod_staticfile_not_handled(request_st * const r, const char * const msg)
 {
     if (r->conf.log_request_handling)
-        log_error(r->conf.errh, __FILE__, __LINE__,
+        log_debug(r->conf.errh, __FILE__, __LINE__,
           "-- NOT handling file as static file, %s forbidden", msg);
     return HANDLER_GO_ON;
 }
