@@ -41,9 +41,13 @@
 #undef USE_OPENSSL_CRYPTO
 #undef USE_GNUTLS_CRYPTO
 #undef USE_NSS_CRYPTO
+#if defined(MBEDTLS_USE_PSA_CRYPTO)
+#include <mbedtls/psa_util.h>
+#else
 #include <mbedtls/ctr_drbg.h>
 #include <mbedtls/entropy.h>
 #endif
+#endif
 #ifdef USE_WOLFSSL_CRYPTO
 #undef USE_OPENSSL_CRYPTO
 #undef USE_GNUTLS_CRYPTO
@@ -219,6 +223,7 @@ static int li_rand_device_bytes (unsigned char *buf, int num)
 static int li_rand_inited;
 static unsigned short xsubi[3];
 #ifdef USE_MBEDTLS_CRYPTO
+#if !defined(MBEDTLS_USE_PSA_CRYPTO)
 #ifdef MBEDTLS_ENTROPY_C
 static mbedtls_entropy_context entropy;
 #ifdef MBEDTLS_CTR_DRBG_C
@@ -226,6 +231,7 @@ static mbedtls_ctr_drbg_context ctr_drbg;
 #endif
 #endif
 #endif
+#endif
 #ifdef USE_WOLFSSL_CRYPTO
 static WC_RNG wolf_globalRNG;
 #endif
@@ -334,6 +340,11 @@ static void li_rand_init (void)
     RAND_seed(xsubi, (int)sizeof(xsubi));
   #endif
   #ifdef USE_MBEDTLS_CRYPTO
+  #if defined(MBEDTLS_USE_PSA_CRYPTO)
+    psa_status_t ps = psa_crypto_init();
+    if (ps != PSA_SUCCESS)
+        ck_bt_abort(__FILE__, __LINE__, "psa_crypto_init() failed");
+  #else
   #ifdef MBEDTLS_ENTROPY_C
     mbedtls_entropy_init(&entropy);
   #ifdef MBEDTLS_CTR_DRBG_C
@@ -346,6 +357,7 @@ static void li_rand_init (void)
   #endif
   #endif
   #endif
+  #endif
   #ifdef USE_NSS_CRYPTO
     if (!NSS_IsInitialized() && NSS_NoDB_Init(NULL) < 0)
         ck_bt_abort(__FILE__, __LINE__, "aborted");
@@ -382,13 +394,28 @@ void li_rand_reseed (void)
   #endif
   #ifdef USE_MBEDTLS_CRYPTO
     if (li_rand_inited) {
+      #if defined(MBEDTLS_USE_PSA_CRYPTO)
+        mbedtls_psa_crypto_free();
+      #else
       #ifdef MBEDTLS_ENTROPY_C
       #ifdef MBEDTLS_CTR_DRBG_C
         mbedtls_ctr_drbg_free(&ctr_drbg);
       #endif
         mbedtls_entropy_free(&entropy);
       #endif
+      #endif
     }
+   #if defined(MBEDTLS_USE_PSA_CRYPTO) && defined(PSA_CRYPTO_DRIVER_TEST)
+    else {
+        /*(kludge to call psa_crypto_init() for sys-crypto-md.h from server.c)*/
+        /*(However, we prefer to defer RNG initialization, and the builtin hash
+         * functions do not require psa_crypto_init(), so skip unless hash func
+         * might use an accelerated crypto driver)*/
+        psa_status_t ps = psa_crypto_init();
+        if (ps != PSA_SUCCESS)
+            ck_bt_abort(__FILE__, __LINE__, "psa_crypto_init() failed");
+    }
+   #endif
   #endif
     if (li_rand_inited) li_rand_init();
 }
@@ -420,12 +447,19 @@ int li_rand_pseudo (void)
     if (i) return i; /*(cond to avoid compiler warning for code after return)*/
   #endif
   #ifdef USE_MBEDTLS_CRYPTO
+  #if defined(MBEDTLS_USE_PSA_CRYPTO)
+    int i;
+    if (0 == mbedtls_psa_get_random(MBEDTLS_PSA_RANDOM_STATE,
+                                    (unsigned char *)&i, sizeof(i)))
+        return i;
+  #else
   #ifdef MBEDTLS_CTR_DRBG_C
     int i;
     if (0 == mbedtls_ctr_drbg_random(&ctr_drbg, (unsigned char *)&i, sizeof(i)))
         return i;
   #endif
   #endif
+  #endif
   #ifdef USE_NSS_CRYPTO
     int i;
     if (SECSuccess == PK11_GenerateRandom((unsigned char *)&i, sizeof(i)))
@@ -466,10 +500,14 @@ void li_rand_pseudo_bytes (unsigned char *buf, int num)
     if (SECSuccess == PK11_GenerateRandom(buf, num)) return;
   #endif
   #ifdef USE_MBEDTLS_CRYPTO
+  #if defined(MBEDTLS_USE_PSA_CRYPTO)
+    if (0 == psa_generate_random(buf, (size_t)num)) return;
+  #else
   #ifdef MBEDTLS_CTR_DRBG_C
     if (0 == mbedtls_ctr_drbg_random(&ctr_drbg, buf, (size_t)num)) return;
   #endif
   #endif
+  #endif
   #ifdef USE_WOLFSSL_CRYPTO
     /* RAND_pseudo_bytes() in WolfSSL is equivalent to RAND_bytes() */
     if (0 == wc_RNG_GenerateBlock(&wolf_globalRNG, (byte *)buf, (word32)num))
@@ -479,6 +517,7 @@ void li_rand_pseudo_bytes (unsigned char *buf, int num)
         buf[i] = li_rand_pseudo() & 0xFF;
 }
 
+#if 0 /*(unused)*/
 int li_rand_bytes (unsigned char *buf, int num)
 {
   #ifdef USE_GNUTLS_CRYPTO /* should use GNUTLS_RND_KEY for long-term keys */
@@ -516,12 +555,17 @@ int li_rand_bytes (unsigned char *buf, int num)
   #endif
   #ifdef USE_MBEDTLS_CRYPTO
   #ifdef MBEDTLS_ENTROPY_C
+  #if defined(MBEDTLS_USE_PSA_CRYPTO)
+    if (0 == psa_generate_random(buf, (size_t)num))
+        return 1;
+  #else
     /*(each call <= MBEDTLS_ENTROPY_BLOCK_SIZE; could implement loop here)*/
     if (num <= MBEDTLS_ENTROPY_BLOCK_SIZE
         && 0 == mbedtls_entropy_func(&entropy, buf, (size_t)num)) {
         return 1;
     }
   #endif
+  #endif
   #endif
     if (1 == li_rand_device_bytes(buf, num)) {
         return 1;
@@ -533,6 +577,7 @@ int li_rand_bytes (unsigned char *buf, int num)
         return 0;
     }
 }
+#endif
 
 void li_rand_cleanup (void)
 {
@@ -549,6 +594,9 @@ void li_rand_cleanup (void)
   #endif
   #endif
   #ifdef USE_MBEDTLS_CRYPTO
+  #if defined(MBEDTLS_USE_PSA_CRYPTO)
+    mbedtls_psa_crypto_free();
+  #else
   #ifdef MBEDTLS_ENTROPY_C
   #ifdef MBEDTLS_CTR_DRBG_C
     mbedtls_ctr_drbg_free(&ctr_drbg);
@@ -556,5 +604,7 @@ void li_rand_cleanup (void)
     mbedtls_entropy_free(&entropy);
   #endif
   #endif
+    li_rand_inited = 0;
+  #endif /* USE_MBEDTLS_CRYPTO */
     ck_memzero(xsubi, sizeof(xsubi));
 }
