@@ -9,6 +9,7 @@ void test_buffer (void);
 void test_burl (void);
 void test_http_header (void);
 void test_http_kv (void);
+void test_http_range (void);
 void test_keyvalue (void);
 void test_request (void);
 
@@ -19,6 +20,7 @@ int main(void) {
     test_burl();
     test_http_header();
     test_http_kv();
+    test_http_range();
     test_keyvalue();
     test_request();
 
